from __future__ import annotations

import tkinter as tk
from tkinter import messagebox

from .client import AlureClient


def activate_with_ui(client: AlureClient) -> None:
    root = tk.Tk()
    root.title("Alure Activation")
    root.geometry("360x220")

    tk.Label(root, text="License key").pack(anchor="w", padx=12, pady=(12, 2))
    license_entry = tk.Entry(root, width=40)
    license_entry.pack(padx=12)

    tk.Label(root, text="Device ID").pack(anchor="w", padx=12, pady=(12, 2))
    device_entry = tk.Entry(root, width=40)
    device_entry.pack(padx=12)

    def on_activate() -> None:
        license_key = license_entry.get().strip()
        device_id = device_entry.get().strip()
        if not license_key or not device_id:
            messagebox.showerror("Error", "Please enter license key and device ID.")
            return
        try:
            result = client.activate(license_key, device_id)
            messagebox.showinfo("Activated", f"Activation ID: {result.activation_id}")
            root.destroy()
        except Exception as exc:
            messagebox.showerror("Activation failed", str(exc))

    tk.Button(root, text="Activate", command=on_activate).pack(pady=16)
    root.mainloop()



class AlureError(Exception):
    """Base error for the Alure SDK."""


class HttpError(AlureError):
    def __init__(self, status_code: int, message: str | None = None):
        super().__init__(message or f"HTTP {status_code}")
        self.status_code = status_code
        self.message = message or ""


class ReceiptError(AlureError):
    pass



from .client import AlureClient
from .storage import FileStorage
from .receipt import ReceiptVerifier, ReceiptValidationResult
from .errors import AlureError, HttpError, ReceiptError

__all__ = [
    "AlureClient",
    "FileStorage",
    "ReceiptVerifier",
    "ReceiptValidationResult",
    "AlureError",
    "HttpError",
    "ReceiptError",
]



from alure_sdk import AlureClient, FileStorage, HttpError


def main() -> None:
    client = AlureClient(
        base_url="http://localhost:3000/api/v1",
        storage=FileStorage("./.alure-test"),
    )

    license_key = input("License key: ").strip()
    device_id = input("Device ID (leave empty for auto): ").strip() or None

    try:
        activation = client.activate(license_key, device_id)
        print("Activation ID:", activation.activation_id)
    except HttpError as exc:
        if exc.status_code == 409:
            print("Activation limit reached. Use an existing receipt or revoke an activation.")
            return
        raise

    verify = client.verify_online()
    print("Verify online:", verify)

    project_id = client.project_id_from_receipt()
    if not project_id:
        print("Missing project_id in receipt.")
        return
    update = client.check_update(project_id=project_id, channel="stable")
    print("Latest version:", update.get("latest_version"))

    asset = update.get("asset") or {}
    asset_id = asset.get("asset_id")
    if asset_id:
        path = client.download_asset(asset_id)
        print("Downloaded to:", path)
    else:
        print("No asset available.")


if __name__ == "__main__":
    main()



from __future__ import annotations

import json
from dataclasses import dataclass
from pathlib import Path


@dataclass
class ReceiptRecord:
    receipt: str
    device_id: str
    activation_id: str | None = None
    project_id: str | None = None


class FileStorage:
    def __init__(self, base_dir: str | Path | None = None):
        self.base_dir = Path(base_dir or Path.home() / ".alure").resolve()
        self.base_dir.mkdir(parents=True, exist_ok=True)
        self._receipt_path = self.base_dir / "receipt.json"

    def save_receipt(self, record: ReceiptRecord) -> None:
        payload = {
            "receipt": record.receipt,
            "device_id": record.device_id,
            "activation_id": record.activation_id,
            "project_id": record.project_id,
        }
        self._receipt_path.write_text(json.dumps(payload, indent=2), encoding="utf-8")

    def load_receipt(self) -> ReceiptRecord | None:
        if not self._receipt_path.exists():
            return None
        payload = json.loads(self._receipt_path.read_text(encoding="utf-8"))
        return ReceiptRecord(
            receipt=payload["receipt"],
            device_id=payload["device_id"],
            activation_id=payload.get("activation_id"),
            project_id=payload.get("project_id"),
        )

    def receipts_path(self) -> Path:
        return self._receipt_path

    def downloads_dir(self) -> Path:
        downloads = self.base_dir / "downloads"
        downloads.mkdir(parents=True, exist_ok=True)
        return downloads



from alure_sdk import AlureClient, FileStorage, HttpError
import urllib.error
from datetime import datetime, timezone
from pathlib import Path


def ensure_license(client: AlureClient) -> bool:
    stored = client.storage.load_receipt()
    if not stored:
        return False

    try:
        online = client.verify_online()
        return bool(online.get("valid"))
    except (HttpError, urllib.error.URLError):
        offline = client.verify_offline(verify_signature=False)
        return offline.valid


def activate_flow(client: AlureClient) -> bool:
    license_key = input("Enter license key: ").strip()
    if not license_key:
        return False
    device_id = input("Device ID (leave empty for auto): ").strip() or None
    try:
        client.activate(license_key, device_id)
        verify = client.verify_online()
        return bool(verify.get("valid"))
    except (HttpError, urllib.error.URLError) as exc:
        print("Activation failed:", exc)
        return False


def is_newer_version(latest: str, current: str) -> bool:
    def parse(value: str) -> list[int]:
        parts = []
        for chunk in value.split("."):
            try:
                parts.append(int(chunk))
            except ValueError:
                parts.append(0)
        return parts

    latest_parts = parse(latest)
    current_parts = parse(current)
    length = max(len(latest_parts), len(current_parts))
    latest_parts += [0] * (length - len(latest_parts))
    current_parts += [0] * (length - len(current_parts))
    return latest_parts > current_parts


def print_expiry(client: AlureClient) -> None:
    stored = client.storage.load_receipt()
    if not stored:
        return
    try:
        payload = client.verifier.parse(stored.receipt)
    except Exception:
        print("License expires: unknown")
        return
    expires_at = payload.get("expires_at")
    if not expires_at:
        print("License expires: never")
        return
    exp_dt = datetime.fromisoformat(expires_at.replace("Z", "+00:00"))
    now = datetime.now(timezone.utc)
    remaining = exp_dt - now
    if remaining.total_seconds() <= 0:
        print("License expired.")
        return
    days = remaining.days
    hours = remaining.seconds // 3600
    print(f"License expires in {days} day(s), {hours} hour(s).")


def main() -> None:
    client = AlureClient(
        base_url="https://api.alure.it/api/v1",
        storage=FileStorage("./.alure-client"),
    )

    if ensure_license(client):
        print("Hello world!")
        print_expiry(client)
        project_id = client.project_id_from_receipt()
        if project_id:
            version_file = Path("./.alure-client/installed_version.txt")
            installed_version = version_file.read_text(encoding="utf-8").strip() if version_file.exists() else ""

            update = client.check_update(project_id=project_id, channel="stable")
            latest = update.get("latest_version")
            print("Latest version:", latest or "n/a")
            if latest and installed_version:
                print("Installed version:", installed_version)
            if latest and installed_version and not is_newer_version(latest, installed_version):
                print("Already up to date.")
                return

            asset = update.get("asset") or {}
            asset_id = asset.get("asset_id")
            if asset_id:
                try:
                    path = client.download_asset(asset_id)
                    print("Downloaded to:", path)
                    if latest:
                        version_file.write_text(latest, encoding="utf-8")
                except HttpError as exc:
                    print("Download failed:", exc)
        return

    print("License missing or invalid. Activation required.")
    if activate_flow(client):
        print("Hello world!")
        print_expiry(client)
    else:
        print("Unable to activate license.")


if __name__ == "__main__":
    main()



from __future__ import annotations

import json
import os
import platform
import urllib.parse
import urllib.request
import urllib.error
import uuid
from dataclasses import dataclass
from typing import Any

from .errors import HttpError
from .receipt import ReceiptVerifier, ReceiptValidationResult
from .storage import FileStorage, ReceiptRecord


@dataclass
class ActivateResponse:
    receipt: str
    activation_id: str
    expires_at: str | None
    grace_period_days: int
    server_time: str


class AlureClient:
    def __init__(
        self,
        base_url: str = "http://localhost:3000/api/v1",
        storage: FileStorage | None = None,
        public_key_pem: str | None = None,
        timeout: int = 10,
    ):
        self.base_url = base_url.rstrip("/")
        self.timeout = timeout
        self.storage = storage or FileStorage()
        self.verifier = ReceiptVerifier(public_key_pem=public_key_pem)

    def default_device_id(self) -> str:
        node = platform.node()
        mac = uuid.getnode()
        try:
            user = os.getlogin()
        except OSError:
            user = ""
        raw = f"{node}-{mac}-{user}"
        return str(uuid.uuid5(uuid.NAMESPACE_DNS, raw))

    def _request(
        self,
        method: str,
        path: str,
        json_body: dict[str, Any] | None = None,
        query: dict[str, Any] | None = None,
        headers: dict[str, str] | None = None,
    ) -> dict[str, Any]:
        url = f"{self.base_url}{path}"
        if query:
            url = f"{url}?{urllib.parse.urlencode(query)}"
        data = json.dumps(json_body).encode("utf-8") if json_body is not None else None
        req = urllib.request.Request(url, data=data, method=method)
        req.add_header("Accept", "application/json")
        if json_body is not None:
            req.add_header("Content-Type", "application/json")
        for key, value in (headers or {}).items():
            req.add_header(key, value)
        try:
            with urllib.request.urlopen(req, timeout=self.timeout) as resp:
                payload = resp.read()
                if not payload:
                    return {}
                return json.loads(payload.decode("utf-8"))
        except urllib.error.HTTPError as exc:
            try:
                detail = exc.read().decode("utf-8")
            except Exception:
                detail = None
            raise HttpError(exc.code, detail) from exc

    def activate(
        self,
        license_key: str,
        device_id: str | None = None,
        app_version: str | None = None,
        device_meta: dict[str, Any] | None = None,
    ) -> ActivateResponse:
        device_id = device_id or self.default_device_id()
        payload: dict[str, Any] = {
            "license_key": license_key,
            "device_id": device_id,
        }
        if app_version:
            payload["app_version"] = app_version
        if device_meta:
            payload["device_meta"] = device_meta
        data = self._request("POST", "/licenses/activate", payload)
        record = ReceiptRecord(
            receipt=data["receipt"],
            device_id=device_id,
            activation_id=data.get("activation_id"),
            project_id=self._extract_project_id(data["receipt"]),
        )
        self.storage.save_receipt(record)
        return ActivateResponse(
            receipt=data["receipt"],
            activation_id=data["activation_id"],
            expires_at=data.get("expires_at"),
            grace_period_days=data["grace_period_days"],
            server_time=data["server_time"],
        )

    def verify_online(self, receipt: str | None = None, device_id: str | None = None) -> dict[str, Any]:
        if receipt is None or device_id is None:
            stored = self.storage.load_receipt()
            if not stored:
                raise HttpError(400, "missing_receipt")
            receipt = receipt or stored.receipt
            device_id = device_id or stored.device_id
        return self._request("POST", "/licenses/verify", {"receipt": receipt, "device_id": device_id})

    def verify_offline(
        self,
        receipt: str | None = None,
        device_id: str | None = None,
        verify_signature: bool = True,
    ) -> ReceiptValidationResult:
        if receipt is None or device_id is None:
            stored = self.storage.load_receipt()
            if not stored:
                return ReceiptValidationResult(valid=False, reason="missing_receipt")
            receipt = receipt or stored.receipt
            device_id = device_id or stored.device_id
        return self.verifier.validate_offline(receipt, device_id, verify_signature=verify_signature)

    def check_update(self, project_id: str, channel: str, current_version: str | None = None) -> dict[str, Any]:
        query = {"project_id": project_id, "channel": channel}
        if current_version:
            query["current_version"] = current_version
        return self._request("GET", "/updates/latest", query=query)

    def project_id_from_receipt(self, receipt: str | None = None) -> str | None:
        if receipt is None:
            stored = self.storage.load_receipt()
            if not stored:
                return None
            receipt = stored.receipt
        try:
            payload = self.verifier.parse(receipt)
            return payload.get("project_id")
        except Exception:
            return None

    def request_download_token(self, receipt: str, device_id: str, asset_id: str) -> dict[str, Any]:
        return self._request(
            "POST",
            "/updates/download-token",
            {"receipt": receipt, "device_id": device_id, "asset_id": asset_id},
        )

    def download_asset(
        self,
        asset_id: str,
        receipt: str | None = None,
        device_id: str | None = None,
        token: str | None = None,
        dest_path: str | None = None,
    ) -> str:
        if token is None:
            if receipt is None or device_id is None:
                stored = self.storage.load_receipt()
                if not stored:
                    raise HttpError(400, "missing_receipt")
                receipt = receipt or stored.receipt
                device_id = device_id or stored.device_id
            token_resp = self.request_download_token(receipt, device_id, asset_id)
            token = token_resp["token"]

        url = f"{self.base_url}/updates/download/{asset_id}?{urllib.parse.urlencode({'token': token})}"
        req = urllib.request.Request(url, method="GET")
        with urllib.request.urlopen(req, timeout=self.timeout) as resp:
            content = resp.read()
            disposition = resp.headers.get("Content-Disposition", "")
            filename = self._filename_from_disposition(disposition) or f"{asset_id}.bin"
            target = dest_path or str(self.storage.downloads_dir() / filename)
            with open(target, "wb") as handle:
                handle.write(content)
            return target

    @staticmethod
    def _filename_from_disposition(disposition: str) -> str | None:
        if "filename=" not in disposition:
            return None
        _, value = disposition.split("filename=", 1)
        return value.strip().strip('"')

    def _extract_project_id(self, receipt: str) -> str | None:
        try:
            payload = self.verifier.parse(receipt)
            return payload.get("project_id")
        except Exception:
            return None



from __future__ import annotations

import base64
import dataclasses
import hashlib
import json
from datetime import datetime, timezone, timedelta
from typing import Any

from .errors import ReceiptError


def _b64url_decode(value: str) -> bytes:
    padded = value.replace("-", "+").replace("_", "/")
    padded += "=" * (-len(padded) % 4)
    return base64.b64decode(padded)


@dataclasses.dataclass
class ReceiptValidationResult:
    valid: bool
    reason: str | None = None
    expires_at: str | None = None
    grace_period_days: int | None = None


class ReceiptVerifier:
    def __init__(self, public_key_pem: str | None = None):
        self.public_key_pem = public_key_pem

    def parse(self, token: str) -> dict[str, Any]:
        parts = token.split(".")
        if len(parts) != 3 or parts[0] != "v1":
            raise ReceiptError("invalid_receipt_format")
        try:
            payload_json = _b64url_decode(parts[1]).decode("utf-8")
            return json.loads(payload_json)
        except Exception as exc:
            raise ReceiptError("invalid_receipt_payload") from exc

    def verify_signature(self, token: str) -> bool:
        if not self.public_key_pem:
            raise ReceiptError("public_key_required")
        try:
            from cryptography.hazmat.primitives import serialization
            from cryptography.hazmat.primitives.asymmetric.ed25519 import Ed25519PublicKey
        except Exception as exc:
            raise ReceiptError("cryptography_missing") from exc

        parts = token.split(".")
        if len(parts) != 3 or parts[0] != "v1":
            return False
        payload_b64 = parts[1].encode("utf-8")
        signature = _b64url_decode(parts[2])
        public_key = serialization.load_pem_public_key(self.public_key_pem.encode("utf-8"))
        if not isinstance(public_key, Ed25519PublicKey):
            raise ReceiptError("invalid_public_key")
        try:
            public_key.verify(signature, payload_b64)
            return True
        except Exception:
            return False

    def validate_offline(
        self,
        token: str,
        device_id: str,
        now: datetime | None = None,
        verify_signature: bool = True,
    ) -> ReceiptValidationResult:
        payload = self.parse(token)
        if verify_signature:
            if not self.verify_signature(token):
                return ReceiptValidationResult(valid=False, reason="invalid_signature")

        device_hash = hashlib.sha256(device_id.encode("utf-8")).hexdigest()
        if payload.get("device_id_hash") != device_hash:
            return ReceiptValidationResult(valid=False, reason="device_mismatch")

        expires_at = payload.get("expires_at")
        grace_days = int(payload.get("grace_period_days") or 0)
        now_dt = now or datetime.now(timezone.utc)

        if expires_at:
            exp_dt = datetime.fromisoformat(expires_at.replace("Z", "+00:00"))
            if now_dt > exp_dt:
                grace_limit = exp_dt + timedelta(days=grace_days)
                if now_dt > grace_limit:
                    return ReceiptValidationResult(valid=False, reason="expired", expires_at=expires_at)
                return ReceiptValidationResult(
                    valid=True,
                    reason="grace_period",
                    expires_at=expires_at,
                    grace_period_days=grace_days,
                )

        return ReceiptValidationResult(valid=True, expires_at=expires_at, grace_period_days=grace_days)
